#!/usr/bin/env python3
"""
Authentication log generator for Splunk security testing
Generates realistic LDAP and RADIUS authentication logs with attack patterns
"""

import time
import random
import datetime
from pathlib import Path

# User accounts for authentication
USERNAMES = [
    'john.doe', 'jane.smith', 'admin', 'administrator', 'service_account',
    'backup_user', 'test.user', 'contractor01', 'dev.account', 'sql_service',
    'web_service', 'ldap_bind', 'monitoring', 'guest', 'temp_user'
]

# Common password attack usernames
ATTACK_USERNAMES = [
    'admin', 'administrator', 'root', 'test', 'guest', 'user', 'sa', 'oracle',
    'postgres', 'mysql', 'ftp', 'www', 'mail', 'email', 'operator', 'manager'
]

# Organizational units for LDAP
ORGANIZATIONAL_UNITS = [
    'ou=Users,dc=company,dc=com',
    'ou=Service Accounts,dc=company,dc=com', 
    'ou=Contractors,dc=company,dc=com',
    'ou=IT,dc=company,dc=com',
    'ou=Finance,dc=company,dc=com',
    'ou=HR,dc=company,dc=com',
    'ou=Sales,dc=company,dc=com'
]

# LDAP servers
LDAP_SERVERS = ['ldap01.company.com', 'ldap02.company.com', 'dc01.company.com']

# RADIUS clients (Network Access Servers)
RADIUS_CLIENTS = [
    {'name': 'WiFi-Controller-01', 'ip': '10.0.1.50', 'type': 'wireless'},
    {'name': 'VPN-Gateway', 'ip': '192.168.1.1', 'type': 'vpn'},
    {'name': 'Switch-Core-01', 'ip': '10.0.1.10', 'type': 'switch'},
    {'name': 'Firewall-01', 'ip': '10.0.1.1', 'type': 'firewall'},
    {'name': 'AP-Floor2-01', 'ip': '10.0.2.100', 'type': 'wireless'},
    {'name': 'Router-Branch-01', 'ip': '192.168.100.1', 'type': 'router'}
]

# Client IP ranges for different access types
WIFI_CLIENTS = ['10.10.1.', '10.10.2.', '10.10.3.']
VPN_CLIENTS = ['172.16.100.', '172.16.101.']
WIRED_CLIENTS = ['192.168.1.', '192.168.2.']

def generate_client_ip(access_type='random'):
    """Generate client IP based on access type"""
    if access_type == 'wifi':
        network = random.choice(WIFI_CLIENTS)
    elif access_type == 'vpn':
        network = random.choice(VPN_CLIENTS)
    elif access_type == 'wired':
        network = random.choice(WIRED_CLIENTS)
    else:
        network = random.choice(WIFI_CLIENTS + VPN_CLIENTS + WIRED_CLIENTS)
    
    return f"{network}{random.randint(1, 254)}"

def generate_mac_address():
    """Generate MAC address"""
    return ':'.join([f'{random.randint(0, 255):02x}' for _ in range(6)])

def generate_ldap_log():
    """Generate LDAP authentication log entry"""
    timestamp = datetime.datetime.now().strftime('%b %d %H:%M:%S')
    ldap_server = random.choice(LDAP_SERVERS)
    
    # Determine if this is an attack (15% chance)
    is_attack = random.random() < 0.15
    
    if is_attack:
        username = random.choice(ATTACK_USERNAMES)
        # Attack patterns
        if random.random() < 0.7:
            result = 'FAIL'
            reason = random.choice([
                'Invalid credentials',
                'Account locked',
                'Account disabled',
                'Bad password'
            ])
        else:
            result = 'SUCCESS'
            reason = 'Authentication successful'
    else:
        username = random.choice(USERNAMES)
        result = random.choices(['SUCCESS', 'FAIL'], weights=[85, 15])[0]
        if result == 'FAIL':
            reason = random.choice([
                'Bad password',
                'Account expired',
                'Password expired',
                'Time restriction'
            ])
        else:
            reason = 'Authentication successful'
    
    client_ip = generate_client_ip()
    connection_id = random.randint(1000, 9999)
    ou = random.choice(ORGANIZATIONAL_UNITS)
    
    # Different LDAP log formats
    log_formats = [
        # OpenLDAP format
        f"{timestamp} {ldap_server} slapd[{random.randint(1000, 9999)}]: conn={connection_id} fd={random.randint(10, 100)} ACCEPT from IP={client_ip}::{random.randint(1024, 65535)}",
        # Microsoft AD format  
        f"{timestamp} {ldap_server} LDAP: {result} - User: {username} - Client: {client_ip} - Reason: {reason}",
        # Detailed bind attempt
        f"{timestamp} {ldap_server} slapd[{random.randint(1000, 9999)}]: conn={connection_id} op={random.randint(0, 10)} BIND dn=\"cn={username},{ou}\" method=128 result={0 if result == 'SUCCESS' else 49}"
    ]
    
    return random.choice(log_formats)

def generate_radius_log():
    """Generate RADIUS authentication log entry"""
    timestamp = datetime.datetime.now().strftime('%b %d %H:%M:%S')
    radius_server = 'radius-01'
    
    # Select NAS (Network Access Server)
    nas = random.choice(RADIUS_CLIENTS)
    nas_ip = nas['ip']
    nas_type = nas['type']
    
    # Generate client info based on NAS type
    if nas_type == 'wireless':
        client_ip = generate_client_ip('wifi')
        calling_station = generate_mac_address().replace(':', '-')
        called_station = f"WiFi-Corp-{random.choice(['5G', '2.4G'])}"
    elif nas_type == 'vpn':
        client_ip = generate_client_ip('vpn')
        calling_station = generate_client_ip()  # External IP for VPN
        called_station = 'VPN-Gateway'
    else:
        client_ip = generate_client_ip('wired')
        calling_station = generate_mac_address()
        called_station = 'Wired-Network'
    
    # Determine authentication result
    is_attack = random.random() < 0.12  # 12% attack rate
    
    if is_attack:
        username = random.choice(ATTACK_USERNAMES)
        auth_result = random.choices(['Access-Accept', 'Access-Reject'], weights=[20, 80])[0]
    else:
        username = random.choice(USERNAMES)
        auth_result = random.choices(['Access-Accept', 'Access-Reject'], weights=[90, 10])[0]
    
    packet_id = random.randint(1, 255)
    session_id = f"{random.randint(10000000, 99999999):08x}"
    
    # RADIUS attributes
    attributes = []
    if auth_result == 'Access-Accept':
        attributes.extend([
            f"Session-Timeout={random.randint(3600, 28800)}",
            f"Idle-Timeout={random.randint(300, 1800)}",
            f"Framed-IP-Address={client_ip}"
        ])
        if nas_type == 'wireless':
            attributes.append(f"Tunnel-Type=VLAN")
            attributes.append(f"Tunnel-Medium-Type=802")
            attributes.append(f"Tunnel-Private-Group-Id={random.randint(10, 100)}")
    else:
        reject_reasons = [
            'Authentication-Failure',
            'User-Account-Disabled', 
            'Invalid-User',
            'Password-Expired',
            'Access-Time-Restriction'
        ]
        attributes.append(f"Reply-Message={random.choice(reject_reasons)}")
    
    # Different RADIUS log formats
    log_formats = [
        # FreeRADIUS format
        f"{timestamp} {radius_server} radiusd[{random.randint(1000, 9999)}]: {auth_result} user '{username}' from client {nas_ip} port {random.randint(0, 100)} cli {calling_station}",
        # Detailed RADIUS format
        f"{timestamp} {radius_server} RADIUS: User={username} NAS-IP={nas_ip} Calling-Station={calling_station} Called-Station={called_station} Result={auth_result} Session={session_id}",
        # Microsoft NPS format
        f"{timestamp} {radius_server} NPS: {auth_result} for user {username} from {calling_station} via {nas_ip} ({nas['name']})"
    ]
    
    base_log = random.choice(log_formats)
    
    # Add attributes for detailed logs
    if random.choice([True, False]) and attributes:
        attr_string = " ".join(attributes)
        base_log += f" [{attr_string}]"
    
    return base_log

def generate_ssh_auth_log():
    """Generate SSH authentication log entry (bonus auth type)"""
    timestamp = datetime.datetime.now().strftime('%b %d %H:%M:%S')
    hostname = random.choice(['web01', 'db01', 'app01', 'jump01'])
    
    is_attack = random.random() < 0.25  # 25% attack rate for SSH
    
    if is_attack:
        username = random.choice(ATTACK_USERNAMES + ['root', 'ubuntu', 'centos'])
        client_ip = f"{random.randint(1, 223)}.{random.randint(0, 255)}.{random.randint(0, 255)}.{random.randint(1, 254)}"  # External IP
        auth_result = random.choices(['success', 'failure'], weights=[10, 90])[0]
    else:
        username = random.choice(USERNAMES)
        client_ip = generate_client_ip()
        auth_result = random.choices(['success', 'failure'], weights=[95, 5])[0]
    
    port = random.randint(1024, 65535)
    ssh_pid = random.randint(1000, 9999)
    
    if auth_result == 'success':
        auth_methods = ['publickey', 'password', 'keyboard-interactive']
        method = random.choice(auth_methods)
        log_entry = f"{timestamp} {hostname} sshd[{ssh_pid}]: Accepted {method} for {username} from {client_ip} port {port} ssh2"
    else:
        if random.random() < 0.7:  # Failed password
            log_entry = f"{timestamp} {hostname} sshd[{ssh_pid}]: Failed password for {username} from {client_ip} port {port} ssh2"
        else:  # Invalid user
            log_entry = f"{timestamp} {hostname} sshd[{ssh_pid}]: Failed password for invalid user {username} from {client_ip} port {port} ssh2"
    
    return log_entry

def main():
    # Create separate log files for different auth types
    ldap_log = Path("/var/log/app/ldap_auth.log")
    radius_log = Path("/var/log/app/radius_auth.log")
    ssh_log = Path("/var/log/app/ssh_auth.log")
    
    for log_file in [ldap_log, radius_log, ssh_log]:
        log_file.parent.mkdir(parents=True, exist_ok=True)
    
    print("Starting authentication log generator...")
    
    while True:
        try:
            # Generate different authentication log types
            auth_type = random.choices(
                ['ldap', 'radius', 'ssh'],
                weights=[35, 35, 30],
                k=1
            )[0]
            
            if auth_type == 'ldap':
                log_entry = generate_ldap_log()
                log_file = ldap_log
            elif auth_type == 'radius':
                log_entry = generate_radius_log()
                log_file = radius_log
            else:
                log_entry = generate_ssh_auth_log()
                log_file = ssh_log
            
            with open(log_file, "a") as f:
                f.write(log_entry + "\n")
            
            print(f"Generated {auth_type} auth log")
            
            # Random delay between 3-20 seconds
            time.sleep(random.uniform(3, 20))
            
        except KeyboardInterrupt:
            print("Stopping authentication log generator...")
            break
        except Exception as e:
            print(f"Error: {e}")
            time.sleep(5)

if __name__ == "__main__":
    main()